@@ -1,6 +1,6 @@
 /* arrayfunc.c -- High-level array functions used by other parts of the shell. */
 
-/* Copyright (C) 2001 Free Software Foundation, Inc.
+/* Copyright (C) 2001-2003 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -27,14 +27,22 @@
 #endif
 #include <stdio.h>
 
+#include "bashintl.h"
+
 #include "shell.h"
+
+#include "shmbutil.h"
+
 #include "builtins/common.h"
 
 extern char *this_command_name;
 extern int last_command_exit_value;
 
 static void quote_array_assignment_chars __P((WORD_LIST *));
-static char *array_value_internal __P((char *, int, int));
+static char *array_value_internal __P((char *, int, int, int *));
+
+/* Standard error message to use when encountering an invalid array subscript */
+char *bash_badsub_errmsg = N_("bad array subscript");
 
 /* **************************************************************** */
 /*								    */
@@ -52,11 +60,16 @@ convert_var_to_array (var)
   ARRAY *array;
 
   oldval = value_cell (var);
-  array = new_array ();
-  array_add_element (array, 0, oldval);
+  array = array_create ();
+  if (oldval)
+    array_insert (array, 0, oldval);
 
   FREE (value_cell (var));
-  var->value = (char *)array;
+  var_setarray (var, array);
+
+  /* these aren't valid anymore */
+  var->dynamic_value = (sh_var_value_func_t *)NULL;
+  var->assign_func = (sh_var_assign_func_t *)NULL;
 
   INVALIDATE_EXPORTSTR (var);
 
@@ -89,7 +102,7 @@ bind_array_variable (name, ind, value)
   else if (readonly_p (entry) || noassign_p (entry))
     {
       if (readonly_p (entry))
-	report_error ("%s: readonly variable", name);
+	err_readonly (name);
       return (entry);
     }
   else if (array_p (entry) == 0)
@@ -98,9 +111,9 @@ bind_array_variable (name, ind, value)
   /* ENTRY is an array variable, and ARRAY points to the value. */
   newval = make_variable_value (entry, value);
   if (entry->assign_func)
-    (*entry->assign_func) (entry, ind, newval);
+    (*entry->assign_func) (entry, newval, ind);
   else
-    array_add_element (array_cell (entry), ind, newval);
+    array_insert (array_cell (entry), ind, newval);
   FREE (newval);
 
   return (entry);
@@ -125,7 +138,7 @@ assign_array_element (name, value)
   if ((ALL_ELEMENT_SUB (sub[0]) && sub[1] == ']') || (sublen <= 1))
     {
       free (vname);
-      report_error ("%s: bad array subscript", name);
+      err_badarraysub (name);
       return ((SHELL_VAR *)NULL);
     }
 
@@ -133,7 +146,7 @@ assign_array_element (name, value)
   if (ind < 0)
     {
       free (vname);
-      report_error ("%s: bad array subscript", name);
+      err_badarraysub (name);
       return ((SHELL_VAR *)NULL);
     }
 
@@ -162,7 +175,7 @@ find_or_make_array_variable (name, check_flags)
   else if (check_flags && (readonly_p (var) || noassign_p (var)))
     {
       if (readonly_p (var))
-	report_error ("%s: readonly variable", name);
+	err_readonly (name);
       return ((SHELL_VAR *)NULL);
     }
   else if (array_p (var) == 0)
@@ -199,9 +212,9 @@ assign_array_var_from_word_list (var, list)
 
   for (a = array_cell (var), l = list, i = 0; l; l = l->next, i++)
     if (var->assign_func)
-      (*var->assign_func) (var, i, l->word->word);
+      (*var->assign_func) (var, l->word->word, i);
     else
-      array_add_element (a, i, l->word->word);
+      array_insert (a, i, l->word->word);
   return var;
 }
 
@@ -222,7 +235,7 @@ assign_array_var_from_string (var, value)
     return var;
 
   /* If this is called from declare_builtin, value[0] == '(' and
-     strchr(value, ')') != 0.  In this case, we need to extract
+     xstrchr(value, ')') != 0.  In this case, we need to extract
      the value from between the parens before going on. */
   if (*value == '(')	/*)*/
     {
@@ -238,7 +251,7 @@ assign_array_var_from_string (var, value)
      shell expansions including pathname generation and word splitting. */
   /* First we split the string on whitespace, using the shell parser
      (ksh93 seems to do this). */
-  list = parse_string_to_word_list (val, "array assign");
+  list = parse_string_to_word_list (val, 1, "array assign");
 
   /* If we're using [subscript]=value, we need to quote each [ and ] to
      prevent unwanted filename expansion. */
@@ -259,14 +272,14 @@ assign_array_var_from_string (var, value)
   /* Now that we are ready to assign values to the array, kill the existing
      value. */
   if (a)
-    empty_array (a);
+    array_flush (a);
 
   for (last_ind = 0, list = nlist; list; list = list->next)
     {
       w = list->word->word;
 
       /* We have a word of the form [ind]=value */
-      if (w[0] == '[')
+      if ((list->word->flags & W_ASSIGNMENT) && w[0] == '[')
 	{
 	  len = skipsubscript (w, 0);
 
@@ -274,9 +287,9 @@ assign_array_var_from_string (var, value)
 	    {
 	      nval = make_variable_value (var, w);
 	      if (var->assign_func)
-		(*var->assign_func) (var, last_ind, nval);
+		(*var->assign_func) (var, nval, last_ind);
 	      else
-		array_add_element (a, last_ind, nval);
+		array_insert (a, last_ind, nval);
 	      FREE (nval);
 	      last_ind++;
 	      continue;
@@ -284,20 +297,20 @@ assign_array_var_from_string (var, value)
 
 	  if (len == 1)
 	    {
-	      report_error ("%s: bad array subscript", w);
+	      err_badarraysub (w);
 	      continue;
 	    }
 
 	  if (ALL_ELEMENT_SUB (w[1]) && len == 2)
 	    {
-	      report_error ("%s: cannot assign to non-numeric index", w);
+	      report_error (_("%s: cannot assign to non-numeric index"), w);
 	      continue;
 	    }
 
 	  ind = array_expand_index (w + 1, len);
 	  if (ind < 0)
 	    {
-	      report_error ("%s: bad array subscript", w);
+	      err_badarraysub (w);
 	      continue;
 	    }
 	  last_ind = ind;
@@ -313,9 +326,9 @@ assign_array_var_from_string (var, value)
 	this_command_name = (char *)NULL;	/* no command name for errors */
       nval = make_variable_value (var, val);
       if (var->assign_func)
-	(*var->assign_func) (var, ind, nval);
+	(*var->assign_func) (var, nval, ind);
       else
-	array_add_element (a, ind, nval);
+	array_insert (a, ind, nval);
       FREE (nval);
       last_ind++;
     }
@@ -340,7 +353,7 @@ quote_array_assignment_chars (list)
       if (l->word == 0 || l->word->word == 0 || l->word->word[0] == '\0')
 	continue;	/* should not happen, but just in case... */
       /* Don't bother if it doesn't look like [ind]=value */
-      if (l->word->word[0] != '[' || strchr (l->word->word, '=') == 0) /* ] */
+      if (l->word->word[0] != '[' || xstrchr (l->word->word, '=') == 0) /* ] */
 	continue;
       s = nword = (char *)xmalloc (strlen (l->word->word) * 2 + 1);
       saw_eq = 0;
@@ -366,14 +379,56 @@ skipsubscript (s, i)
      int i;
 {
   int count, c;
+#if defined (HANDLE_MULTIBYTE)
+  mbstate_t state, state_bak;
+  size_t slength, mblength;
+  size_t mb_cur_max;
+#endif
 
-  for (count = 1; count && (c = s[++i]); )
+#if defined (HANDLE_MULTIBYTE)
+  memset (&state, '\0', sizeof (mbstate_t));
+  slength = strlen (s + i);
+  mb_cur_max = MB_CUR_MAX;
+#endif
+  
+  count = 1;
+  while (count)
     {
-      if (c == '[')
+      /* Advance one (possibly multibyte) character in S starting at I. */
+#if defined (HANDLE_MULTIBYTE)
+      if (mb_cur_max > 1)
+	{
+	  state_bak = state;
+	  mblength = mbrlen (s + i, slength, &state);
+
+	  if (MB_INVALIDCH (mblength))
+	    {
+	      state = state_bak;
+	      i++;
+	      slength--;
+	    }
+	  else if (MB_NULLWCH (mblength))
+	    return i;
+	  else
+	    {
+	      i += mblength;
+	      slength -= mblength;
+	    }
+	}
+      else
+#endif
+      ++i;
+
+      c = s[i];
+
+      if (c == 0)
+        break;
+      else if (c == '[')
 	count++;
       else if (c == ']')
 	count--;
     }
+
   return i;
 }
 
@@ -392,25 +447,25 @@ unbind_array_element (var, sub)
   len = skipsubscript (sub, 0);
   if (sub[len] != ']' || len == 0)
     {
-      builtin_error ("%s[%s: bad array subscript", var->name, sub);
+      builtin_error ("%s[%s: %s", var->name, sub, _(bash_badsub_errmsg));
       return -1;
     }
   sub[len] = '\0';
 
   if (ALL_ELEMENT_SUB (sub[0]) && sub[1] == 0)
     {
-      makunbound (var->name, shell_variables);
+      unbind_variable (var->name);
       return (0);
     }
   ind = array_expand_index (sub, len+1);
   if (ind < 0)
     {
-      builtin_error ("[%s]: bad array subscript", sub);
+      builtin_error ("[%s]: %s", sub, _(bash_badsub_errmsg));
       return -1;
     }
-  ae = array_delete_element (array_cell (var), ind);
+  ae = array_remove (array_cell (var), ind);
   if (ae)
-    destroy_array_element (ae);
+    array_dispose_element (ae);
   return 0;
 }
 
@@ -423,10 +478,7 @@ print_array_assignment (var, quoted)
 {
   char *vstr;
 
-  if (quoted)
-    vstr = quoted_array_assignment_string (array_cell (var));
-  else
-    vstr = array_to_assignment_string (array_cell (var));
+  vstr = array_to_assign (array_cell (var), quoted);
 
   if (vstr == 0)
     printf ("%s=%s\n", var->name, quoted ? "'()'" : "()");
@@ -451,7 +503,7 @@ valid_array_reference (name)
   char *t;
   int r, len;
 
-  t = strchr (name, '[');	/* ] */
+  t = xstrchr (name, '[');	/* ] */
   if (t)
     {
       *t = '\0';
@@ -509,14 +561,24 @@ array_variable_name (s, subp, lenp)
   char *t, *ret;
   int ind, ni;
 
-  t = strchr (s, '[');
+  t = xstrchr (s, '[');
   if (t == 0)
-    return ((char *)NULL);
+    {
+      if (subp)
+      	*subp = t;
+      if (lenp)
+	*lenp = 0;
+      return ((char *)NULL);
+    }
   ind = t - s;
   ni = skipsubscript (s, ind);
   if (ni <= ind + 1 || s[ni] != ']')
     {
-      report_error ("%s: bad array subscript", s);
+      err_badarraysub (s);
+      if (subp)
+      	*subp = t;
+      if (lenp)
+	*lenp = 0;
       return ((char *)NULL);
     }
 
@@ -549,16 +611,18 @@ array_variable_part (s, subp, lenp)
   var = find_variable (t);
 
   free (t);
-  return var;
+  return (var == 0 || invisible_p (var)) ? (SHELL_VAR *)0 : var;
 }
 
 /* Return a string containing the elements in the array and subscript
    described by S.  If the subscript is * or @, obeys quoting rules akin
-   to the expansion of $* and $@ including double quoting. */
+   to the expansion of $* and $@ including double quoting.  If RTYPE
+   is non-null it gets 1 if the array reference is name[@] or name[*]
+   and 0 otherwise. */
 static char *
-array_value_internal (s, quoted, allow_all)
+array_value_internal (s, quoted, allow_all, rtype)
      char *s;
-     int quoted, allow_all;
+     int quoted, allow_all, *rtype;
 {
   int len;
   arrayind_t ind;
@@ -568,22 +632,30 @@ array_value_internal (s, quoted, allow_all)
 
   var = array_variable_part (s, &t, &len);
 
+  /* Expand the index, even if the variable doesn't exist, in case side
+     effects are needed, like ${w[i++]} where w is unset. */
+#if 0
   if (var == 0)
     return (char *)NULL;
+#endif
+
+  if (len == 0)
+    return ((char *)NULL);	/* error message already printed */
 
   /* [ */
   if (ALL_ELEMENT_SUB (t[0]) && t[1] == ']')
     {
+      if (rtype)
+	*rtype = 1;
       if (allow_all == 0)
 	{
-	  report_error ("%s: bad array subscript", s);
+	  err_badarraysub (s);
 	  return ((char *)NULL);
 	}
+      else if (var == 0)
+	return ((char *)NULL);
       else if (array_p (var) == 0)
-	{
-	  l = (WORD_LIST *)NULL;
-	  l = add_string_to_list (value_cell (var), l);
-	}
+	l = add_string_to_list (value_cell (var), (WORD_LIST *)NULL);
       else
 	{
 	  l = array_to_word_list (array_cell (var));
@@ -604,17 +676,26 @@ array_value_internal (s, quoted, allow_all)
     }
   else
     {
+      if (rtype)
+	*rtype = 0;
       ind = array_expand_index (t, len);
       if (ind < 0)
 	{
-	  report_error ("%s: bad array subscript", var->name);
+	  if (var)
+	    err_badarraysub (var->name);
+	  else
+	    {
+	      t[-1] = '\0';
+	      err_badarraysub (s);
+	      t[-1] = '[';	/* ] */
+	    }
 	  return ((char *)NULL);
 	}
+      if (var == 0)
+	return ((char *)NULL);
       if (array_p (var) == 0)
-	return (ind == 0 ? savestring (value_cell (var)) : (char *)NULL);
+	return (ind == 0 ? value_cell (var) : (char *)NULL);
       retval = array_reference (array_cell (var), ind);
-      if (retval)
-	retval = quote_escapes (retval);
     }
 
   return retval;
@@ -623,11 +704,11 @@ array_value_internal (s, quoted, allow_all)
 /* Return a string containing the elements described by the array and
    subscript contained in S, obeying quoting for subscripts * and @. */
 char *
-array_value (s, quoted)
+array_value (s, quoted, rtype)
      char *s;
-     int quoted;
+     int quoted, *rtype;
 {
-  return (array_value_internal (s, quoted, 1));
+  return (array_value_internal (s, quoted, 1, rtype));
 }
 
 /* Return the value of the array indexing expression S as a single string.
@@ -635,11 +716,48 @@ array_value (s, quoted)
    by other parts of the shell such as the arithmetic expression evaluator
    in expr.c. */
 char *
-get_array_value (s, allow_all)
+get_array_value (s, allow_all, rtype)
      char *s;
-     int allow_all;
+     int allow_all, *rtype;
 {
-  return (array_value_internal (s, 0, allow_all));
+  return (array_value_internal (s, 0, allow_all, rtype));
 }
 
+char *
+array_keys (s, quoted)
+     char *s;
+     int quoted;
+{
+  int len;
+  char *retval, *t, *temp;
+  WORD_LIST *l;
+  SHELL_VAR *var;
+
+  var = array_variable_part (s, &t, &len);
+
+  /* [ */
+  if (var == 0 || ALL_ELEMENT_SUB (t[0]) == 0 || t[1] != ']')
+    return (char *)NULL;
+
+  if (array_p (var) == 0)
+    l = add_string_to_list ("0", (WORD_LIST *)NULL);
+  else
+    {
+      l = array_keys_to_word_list (array_cell (var));
+      if (l == (WORD_LIST *)NULL)
+        return ((char *) NULL);
+    }
+
+  if (t[0] == '*' && (quoted & (Q_HERE_DOCUMENT|Q_DOUBLE_QUOTES)))
+    {
+      temp = string_list_dollar_star (l);
+      retval = quote_string (temp);
+      free (temp);
+    }
+  else	/* ${!name[@]} or unquoted ${!name[*]} */
+    retval = string_list_dollar_at (l, quoted);
+
+  dispose_words (l);
+  return retval;
+}
 #endif /* ARRAY_VARS */
