@@ -1,7 +1,7 @@
 /* make_cmd.c -- Functions for making instances of the various
    parser constructs. */
 
-/* Copyright (C) 1989 Free Software Foundation, Inc.
+/* Copyright (C) 1989-2003 Free Software Foundation, Inc.
 
 This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -23,7 +23,7 @@ Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */
 
 #include <stdio.h>
 #include "bashtypes.h"
-#ifndef _MINIX
+#if !defined (_MINIX) && defined (HAVE_SYS_FILE_H)
 #  include <sys/file.h>
 #endif
 #include "filecntl.h"
@@ -32,37 +32,61 @@ Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */
 #  include <unistd.h>
 #endif
 
+#include "bashintl.h"
+
 #include "syntax.h"
 #include "command.h"
 #include "general.h"
 #include "error.h"
 #include "flags.h"
 #include "make_cmd.h"
+#include "dispose_cmd.h"
 #include "variables.h"
 #include "subst.h"
 #include "input.h"
+#include "ocache.h"
 #include "externs.h"
 
 #if defined (JOB_CONTROL)
 #include "jobs.h"
 #endif
 
+#include "shmbutil.h"
+
 extern int line_number, current_command_line_count;
 extern int last_command_exit_value;
 
-static COMMAND *make_for_or_select __P((enum command_type, WORD_DESC *, WORD_LIST *, COMMAND *));
+/* Object caching */
+sh_obj_cache_t wdcache = {0, 0, 0};
+sh_obj_cache_t wlcache = {0, 0, 0};
+
+#define WDCACHESIZE	60
+#define WLCACHESIZE	60
+
+static COMMAND *make_for_or_select __P((enum command_type, WORD_DESC *, WORD_LIST *, COMMAND *, int));
 #if defined (ARITH_FOR_COMMAND)
 static WORD_LIST *make_arith_for_expr __P((char *));
 #endif
 static COMMAND *make_until_or_while __P((enum command_type, COMMAND *, COMMAND *));
 
+void
+cmd_init ()
+{
+  ocache_create (wdcache, WORD_DESC, WDCACHESIZE);
+  ocache_create (wlcache, WORD_LIST, WLCACHESIZE);
+}
+
 WORD_DESC *
 make_bare_word (string)
      const char *string;
 {
   WORD_DESC *temp;
-
+#if 0
   temp = (WORD_DESC *)xmalloc (sizeof (WORD_DESC));
+#else
+  ocache_alloc (wdcache, WORD_DESC, temp);
+#endif
+
   if (*string)
     temp->word = savestring (string);
   else
@@ -80,11 +104,16 @@ make_word_flags (w, string)
      WORD_DESC *w;
      const char *string;
 {
-  register const char *s;
+  register int i;
+  size_t slen;
+  DECLARE_MBSTATE;
 
-  for (s = string; *s; s++)
-    switch (*s)
-      {
+  i = 0;
+  slen = strlen (string);
+  while (i < slen)
+    {
+      switch (string[i])
+	{
 	case '$':
 	  w->flags |= W_HASDOLLAR;
 	  break;
@@ -95,7 +124,11 @@ make_word_flags (w, string)
 	case '"':
 	  w->flags |= W_QUOTED;
 	  break;
-      }
+	}
+
+      ADVANCE_CHAR (string, slen, i);
+    }
+
   return (w);
 }
 
@@ -128,25 +161,16 @@ make_word_list (word, wlink)
 {
   WORD_LIST *temp;
 
+#if 0
   temp = (WORD_LIST *)xmalloc (sizeof (WORD_LIST));
+#else
+  ocache_alloc (wlcache, WORD_LIST, temp);
+#endif
   temp->word = word;
   temp->next = wlink;
   return (temp);
 }
 
-WORD_LIST *
-add_string_to_list (string, list)
-     char *string;
-     WORD_LIST *list;
-{
-  WORD_LIST *temp;
-
-  temp = (WORD_LIST *)xmalloc (sizeof (WORD_LIST));
-  temp->word = make_word (string);
-  temp->next = list;
-  return (temp);
-}
-
 COMMAND *
 make_command (type, pointer)
      enum command_type type;
@@ -177,39 +201,43 @@ command_connect (com1, com2, connector)
 }
 
 static COMMAND *
-make_for_or_select (type, name, map_list, action)
+make_for_or_select (type, name, map_list, action, lineno)
      enum command_type type;
      WORD_DESC *name;
      WORD_LIST *map_list;
      COMMAND *action;
+     int lineno;
 {
   FOR_COM *temp;
 
   temp = (FOR_COM *)xmalloc (sizeof (FOR_COM));
   temp->flags = 0;
   temp->name = name;
+  temp->line = lineno;
   temp->map_list = map_list;
   temp->action = action;
   return (make_command (type, (SIMPLE_COM *)temp));
 }
 
 COMMAND *
-make_for_command (name, map_list, action)
+make_for_command (name, map_list, action, lineno)
      WORD_DESC *name;
      WORD_LIST *map_list;
      COMMAND *action;
+     int lineno;
 {
-  return (make_for_or_select (cm_for, name, map_list, action));
+  return (make_for_or_select (cm_for, name, map_list, action, lineno));
 }
 
 COMMAND *
-make_select_command (name, map_list, action)
+make_select_command (name, map_list, action, lineno)
      WORD_DESC *name;
      WORD_LIST *map_list;
      COMMAND *action;
+     int lineno;
 {
 #if defined (SELECT_COMMAND)
-  return (make_for_or_select (cm_select, name, map_list, action));
+  return (make_for_or_select (cm_select, name, map_list, action, lineno));
 #else
   last_command_exit_value = 2;
   return ((COMMAND *)NULL);
@@ -222,16 +250,21 @@ make_arith_for_expr (s)
      char *s;
 {
   WORD_LIST *result;
-  WORD_DESC *w;
+  WORD_DESC *wd;
 
   if (s == 0 || *s == '\0')
     return ((WORD_LIST *)NULL);
-  w = make_word (s);
-  result = make_word_list (w, (WORD_LIST *)NULL);
+  wd = make_word (s);
+  wd->flags |= W_NOGLOB|W_NOSPLIT|W_QUOTED;	/* no word splitting or globbing */
+  result = make_word_list (wd, (WORD_LIST *)NULL);
   return result;
 }
 #endif
 
+/* Note that this function calls dispose_words on EXPRS, since it doesn't
+   use the word list directly.  We free it here rather than at the caller
+   because no other function in this file requires that the caller free
+   any arguments. */
 COMMAND *
 make_arith_for_command (exprs, action, lineno)
      WORD_LIST *exprs;
@@ -282,10 +315,10 @@ make_arith_for_command (exprs, action, lineno)
   if (nsemi != 3)
     {
       if (nsemi < 3)
-	parser_error (lineno, "syntax error: arithmetic expression required");
+	parser_error (lineno, _("syntax error: arithmetic expression required"));
       else
-	parser_error (lineno, "syntax error: `;' unexpected");
-      parser_error (lineno, "syntax error: `((%s))'", exprs->word->word);
+	parser_error (lineno, _("syntax error: `;' unexpected"));
+      parser_error (lineno, _("syntax error: `((%s))'"), exprs->word->word);
       last_command_exit_value = 2;
       return ((COMMAND *)NULL);
     }
@@ -298,8 +331,10 @@ make_arith_for_command (exprs, action, lineno)
   temp->step = step ? step : make_arith_for_expr ("1");
   temp->action = action;
 
+  dispose_words (exprs);
   return (make_command (cm_arith_for, (SIMPLE_COM *)temp));
 #else
+  dispose_words (exprs);
   last_command_exit_value = 2;
   return ((COMMAND *)NULL);
 #endif /* ARITH_FOR_COMMAND */
@@ -317,7 +352,7 @@ make_group_command (command)
 }
 
 COMMAND *
-make_case_command (word, clauses)
+make_case_command (word, clauses, lineno)
      WORD_DESC *word;
      PATTERN_LIST *clauses;
 {
@@ -325,6 +360,7 @@ make_case_command (word, clauses)
 
   temp = (CASE_COM *)xmalloc (sizeof (CASE_COM));
   temp->flags = 0;
+  temp->line = lineno;
   temp->word = word;
   temp->clauses = REVERSE_LIST (clauses, PATTERN_LIST *);
   return (make_command (cm_case, (SIMPLE_COM *)temp));
@@ -490,13 +526,8 @@ make_simple_command (element, command)
     command = make_bare_simple_command ();
 
   if (element.word)
-    {
-      WORD_LIST *tw = (WORD_LIST *)xmalloc (sizeof (WORD_LIST));
-      tw->word = element.word;
-      tw->next = command->value.Simple->words;
-      command->value.Simple->words = tw;
-    }
-  else
+    command->value.Simple->words = make_word_list (element.word, command->value.Simple->words);
+  else if (element.redirect)
     {
       REDIRECT *r = element.redirect;
       /* Due to the way <> is implemented, there may be more than a single
@@ -526,7 +557,7 @@ make_here_document (temp)
   if (temp->instruction != r_deblank_reading_until &&
       temp->instruction != r_reading_until)
     {
-      internal_error ("make_here_document: bad instruction type %d", temp->instruction);
+      internal_error (_("make_here_document: bad instruction type %d"), temp->instruction);
       return;
     }
 
@@ -624,7 +655,12 @@ make_redirection (source, instruction, dest_and_filename)
      enum r_instruction instruction;
      REDIRECTEE dest_and_filename;
 {
-  REDIRECT *temp = (REDIRECT *)xmalloc (sizeof (REDIRECT));
+  REDIRECT *temp;
+  WORD_DESC *w;
+  int wlen;
+  intmax_t lfd;
+
+  temp = (REDIRECT *)xmalloc (sizeof (REDIRECT));
 
   /* First do the common cases. */
   temp->redirector = source;
@@ -657,15 +693,41 @@ make_redirection (source, instruction, dest_and_filename)
 
     case r_deblank_reading_until: 	/* <<-foo */
     case r_reading_until:		/* << foo */
+    case r_reading_string:		/* <<< foo */
     case r_close_this:			/* <&- */
     case r_duplicating_input:		/* 1<&2 */
     case r_duplicating_output:		/* 1>&2 */
+      break;
+
+    /* the parser doesn't pass these. */
+    case r_move_input:			/* 1<&2- */
+    case r_move_output:			/* 1>&2- */
+    case r_move_input_word:		/* 1<&$foo- */
+    case r_move_output_word:		/* 1>&$foo- */
+      break;
+
+    /* The way the lexer works we have to do this here. */
     case r_duplicating_input_word:	/* 1<&$foo */
     case r_duplicating_output_word:	/* 1>&$foo */
+      w = dest_and_filename.filename;
+      wlen = strlen (w->word) - 1;
+      if (w->word[wlen] == '-')		/* Yuck */
+        {
+          w->word[wlen] = '\0';
+	  if (all_digits (w->word) && legal_number (w->word, &lfd) && lfd == (int)lfd)
+	    {
+	      dispose_word (w);
+	      temp->instruction = (instruction == r_duplicating_input_word) ? r_move_input : r_move_output;
+	      temp->redirectee.dest = lfd;
+	    }
+	  else
+	    temp->instruction = (instruction == r_duplicating_input_word) ? r_move_input_word : r_move_output_word;
+        }
+          
       break;
 
     default:
-      programming_error ("make_redirection: redirection instruction `%d' out of range", instruction);
+      programming_error (_("make_redirection: redirection instruction `%d' out of range"), instruction);
       abort ();
       break;
     }
@@ -679,6 +741,11 @@ make_function_def (name, command, lineno, lstart)
      int lineno, lstart;
 {
   FUNCTION_DEF *temp;
+#if defined (ARRAY_VARS)
+  SHELL_VAR *bash_source_v;
+  ARRAY *bash_source_a;
+  char *t;
+#endif
 
   temp = (FUNCTION_DEF *)xmalloc (sizeof (FUNCTION_DEF));
   temp->command = command;
@@ -686,6 +753,17 @@ make_function_def (name, command, lineno, lstart)
   temp->line = lineno;
   temp->flags = 0;
   command->line = lstart;
+
+  /* Information used primarily for debugging. */
+  temp->source_file = 0;
+#if defined (ARRAY_VARS)
+  GET_ARRAY_FROM_VAR ("BASH_SOURCE", bash_source_v, bash_source_a);
+  if (bash_source_a && array_num_elements (bash_source_a) > 0)
+    temp->source_file = array_reference (bash_source_a, 0);
+#endif
+  bind_function_def (name->word, temp);
+
+  temp->source_file = 0;
   return (make_command (cm_function_def, (SIMPLE_COM *)temp));
 }
 
